/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

// Tile sizes, IMAGE_INNER_TILE is the algorithm tile size, IMAGE_OUTER_TILE is the tile 
// size including guard band. STAT_TILE is the tile size used to gather statistics in the 
// first pass, it is smaller to reduce register pressure
#define STAT_TILE        8 
#define IMAGE_INNER_TILE 16
#define IMAGE_OUTER_TILE 32

#define OUTER_COMPUTE_RATIO ((IMAGE_OUTER_TILE + IMAGE_INNER_TILE - 1) / IMAGE_INNER_TILE)
#define RING_SIZE (IMAGE_OUTER_TILE - IMAGE_INNER_TILE) / 2

// Filtering mode: during (rotated) filtering and reprojection we can use either bilinear or 
// nearest filtering when computing the color at a given position. Bilinear reprojection is 
// disabled by default because it is slow and does not improve quality significantly.
#define FILTER_BILINEAR
//#define REPROJECT_BILINEAR

// Layer selection is slightly different from CPU version. In the CoC region [-16, 16] 
// pixels we insert the number layers given by COC_LAYERS and outside of that region we 
// use UNIFORM_LAYERS layers. An additional layer is used to handle background / clears
#define UNIFORM_LAYERS 2
#define COC_LAYERS 14
#define MAX_LAYERS (COC_LAYERS + UNIFORM_LAYERS + 1)
#define DOF_PART_MIN -16.0f
#define DOF_PART_MAX 16.0f

// Precision for fixed point color & alpha/weight values.  Should be ported to floats (HDR) 
// if/when hardware support float atomics. Note: The floating point filtering is done without 
// renormalizing, so very large intensities may suffer from precision problems. Also, clamping 
// is handled per-component, and may cause color shift.
#define COLOR_SCALE (1 << 12)
#define COLOR_CLAMP (1 << 24)
#define WU_SCALE (1 << 4)
#define WU_SHIFT 16
#define WU_MASK ((1 << WU_SHIFT)-1)

// Misc defines
#define GPU_EPS 1E-8
#define GPU_FLT_MAX 10000000.0f
#define GPU_PI 3.141592653589793f
#define GPU_DEPTH_EPS 1E-3
