#pragma once

/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include "common.h"
#include "config.h"

class CCameraSys
{
public:
	CCameraSys(const float C0, const float C1) : m_C0(C0), m_C1(C1) { m_maxBlur = GetConfig()->MaxMotion; }

	inline float ShearUV(float w) { return m_C0 / w + m_C1; }

	inline float ShearUVInv(float s)
	{
		if (s > m_C1)
		{
			return FLT_MAX;
		}
		return m_C0 / (s - m_C1);
	}

	inline float ShearUVClamp(float w)
	{
		float s = m_C0 / w + m_C1;
		if (fabs(s) > m_maxBlur)
		{
			float scalefac = m_maxBlur / fabs(s);
			s = s * scalefac;
		}
		return s;
	}

	bool hasDof() { return fabs(m_C1) > 0.000001f; }

	inline float2 PixelMotion(const SampleInfo &si)
	{
		float2 res(0, 0);

		// Compute motion vector in xy plane expressed in pixels
		res.x = (-si.imageX * si.vz + si.vx) / (si.w + si.vz);
		res.y = (si.imageY * si.vz - si.vy) / (si.w + si.vz);

		// Normalize
		float len = sqrt(res.x * res.x + res.y * res.y);
		if (len > m_maxBlur)
		{
			res *= m_maxBlur / len;
		}
		return res;
	}

	float GetC0() { return m_C0; }
	float GetC1() { return m_C1; }

private:
	float m_C0;
	float m_C1;
	float m_maxBlur;
};
