#pragma once

/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include <cstring>

///////////////////////////////////////////////////////////////////////////
//					Output Image Types
///////////////////////////////////////////////////////////////////////////
static const int MAX_STRING = 20;

///////////////////////////////////////////////////////////////////////////
//					Reconstruction Algorithms
///////////////////////////////////////////////////////////////////////////

enum AlgoTypes
{
	RECON_GAUSS = 0,
	RECON_5D,
	RECON_5D_GPU,
	NUM_ALGOS
};

enum FilterType
{
	FILTER_NEAREST,
	FILTER_BILINEAR
};

struct StrEnumTuple
{
	StrEnumTuple(unsigned int EnumIn, const char *NameIn)
	{
		Enum = EnumIn;
		strncpy_s(Name, MAX_STRING, NameIn, MAX_STRING);
	};
	unsigned int Enum;
	char Name[MAX_STRING];
};

///////////////////////////////////////////////////////////////////////////
//						Global Configuration
///////////////////////////////////////////////////////////////////////////

class Config
{
public:
	Config();
	static void PrintUsage();
	void Print();
	void Parse(int argc, const char *argv[]);

	inline AlgoTypes &GetAlgo() { return Algo; }

	inline const char *GetAlgoName() { return ALGO_LIST[Algo].Name; }

	int DepthLayers;
	int RecWin;
	int BlockSize;
	int DumpLayer;
	float MaxMotion;
	float sigma_x;
	float sigma_a;
	float sigma_t;
	AlgoTypes Algo;
	FilterType LookupFilter;

private:
	static const StrEnumTuple ALGO_LIST[NUM_ALGOS];
};

#define GET_FLOAT(Z, X, Y)                   \
	if (X->Attribute(#Y) != nullptr)         \
	{                                        \
		Z.Y = (float)atof(X->Attribute(#Y)); \
	}

#define GET_INT(Z, X, Y)              \
	if (X->Attribute(#Y) != nullptr)  \
	{                                 \
		Z.Y = atoi(X->Attribute(#Y)); \
	}

#define GET_BOOL(Z, X, Y)                                  \
	if (X->Attribute(#Y) != nullptr)                       \
	{                                                      \
		if (_stricmp(X->Attribute(#Y), "true") == 0)       \
			Z.Y = true;                                    \
		else if (_stricmp(X->Attribute(#Y), "false") == 0) \
			Z.Y = false;                                   \
	}

Config *InitConfig(int argc, const char *argv[]);
Config *GetConfig();