#pragma once

/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include "../common.h"

class CMeasurement
{
public:
	CMeasurement(ID3D11Device *pDevice)
	{
		// Creating GPU queries
		D3D11_QUERY_DESC desc0 = {D3D11_QUERY_TIMESTAMP, 0};
		D3D11_QUERY_DESC desc1 = {D3D11_QUERY_TIMESTAMP_DISJOINT, 0};
		pDevice->CreateQuery(&desc0, &m_pGPUQuery[0]);
		pDevice->CreateQuery(&desc0, &m_pGPUQuery[1]);
		pDevice->CreateQuery(&desc1, &m_pGPUQuery[2]);
	}

	~CMeasurement()
	{
		SAFE_RELEASE(m_pGPUQuery[0]);
		SAFE_RELEASE(m_pGPUQuery[1]);
		SAFE_RELEASE(m_pGPUQuery[2]);
	}

	void BeginMeasurement(ID3D11DeviceContext *pd3dImmediateContext)
	{
		pd3dImmediateContext->Begin(m_pGPUQuery[2]);
		pd3dImmediateContext->End(m_pGPUQuery[0]);
	}

	FLOAT EndMeasurement(ID3D11DeviceContext *pd3dImmediateContext)
	{
		pd3dImmediateContext->End(m_pGPUQuery[2]);
		pd3dImmediateContext->End(m_pGPUQuery[1]);

		// Get GPU counters
		UINT64 queryTimeA, queryTimeB;
		D3D11_QUERY_DATA_TIMESTAMP_DISJOINT queryTimeData;

		while ((S_OK != pd3dImmediateContext->GetData(m_pGPUQuery[0], &queryTimeA, sizeof(UINT64), 0))
		       || (S_OK != pd3dImmediateContext->GetData(m_pGPUQuery[1], &queryTimeB, sizeof(UINT64), 0))
		       || (S_OK
		           != pd3dImmediateContext->GetData(m_pGPUQuery[2], &queryTimeData,
		                                            sizeof(D3D11_QUERY_DATA_TIMESTAMP_DISJOINT), 0)))
		{
		}

		FLOAT runtime;
		// Calculate time
		if (queryTimeData.Disjoint == 0)
		{
			UINT64 deltaTimeTicks = queryTimeB - queryTimeA;
			runtime = (FLOAT)deltaTimeTicks * 1000.0f / (FLOAT)queryTimeData.Frequency;
		}
		else
		{
			runtime = 0.0f;
		}

		return runtime;
	}

protected:
	ID3D11Query *m_pGPUQuery[3];
};
