/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include "image.h"
#include "lodepng/lodepng.h"

#include <iostream>
using namespace std;

float4 Image::Tonemap(const float4 col, const float exposure, const float gamma)
{
	return saturate(pow(saturate_low(col) * pow(2.0f, exposure), 1.0f / gamma)); 
}

bool Image::SavePNG(const std::string &filename, const float exposure, const float gamma)
{
	cout << "Writing " << filename << endl;
	const int channels = 4; // RGBA

	std::vector<unsigned char> image;
	image.resize(m_width * m_height * 4);
	for (unsigned int y = 0; y < m_height; y++)
	{
		for (unsigned int x = 0; x < m_width; x++)
		{
			float4 c = Tonemap(m_buffer[x + y * m_width], exposure, gamma);
			image[4 * m_width * y + 4 * x + 0] = static_cast<unsigned char>(255 * c.get(0));
			image[4 * m_width * y + 4 * x + 1] = static_cast<unsigned char>(255 * c.get(1));
			image[4 * m_width * y + 4 * x + 2] = static_cast<unsigned char>(255 * c.get(2));
			image[4 * m_width * y + 4 * x + 3] = static_cast<unsigned char>(255 * c.get(3));
		}
	}

	// Encode the image
	unsigned int error = lodepng::encode(filename, image, m_width, m_height);

	// if there's an error, display it
	if (error)
	{
		std::cout << "encoder error " << error << ": " << lodepng_error_text(error) << std::endl;
		return false;
	}

	return true;
}
