#pragma once

/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

struct FilterInfo
{
	float sigmax;
	float sigmay;
	float sigmau;
	float sigmav;
	float sigmat;
	float shearxu;
	float shearyv;
	float shearxt;
	float angle;
};

class Partition
{
public:
	Partition(float _rangeMin, float _rangeMax);

	void AddSample(const float depth, const float vx, const float vy, const float shear, const float weight);

	void ComputeFilter(const float clampVal, const float s_x, const float s_a, const float s_t);

	FilterInfo GetFilterInfo() const { return fi; }

	inline bool InRange(float w) const { return rangeMin <= w && w <= rangeMax; }

	inline bool InTrimmedRange(float w) const { return dmin <= w && w <= dmax; }

	inline bool IsEmpty() const { return sampleWeight == 0.0f; }

	inline bool IsFullyBehindSample(float w) const { return dmin > w; }

	inline bool IsInFrontOfSample(float w) const { return dmax < w; }

	void Clear();

private:
	float ComputeAngle(const float x, const float y) const;

	float rangeMin; // Min and max range for partition
	float rangeMax;

	float dmin; // sample min/max depth
	float dmax;
	float smin; // sample min/max shear
	float smax;

	// Sample motion vector
	float xsum;  // for average
	float x2sum; // for std dev
	float ysum;
	float y2sum;

	float sampleWeight;
	FilterInfo fi;
};
