#pragma once

/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include <d3d11.h>

#include "reconstruct.h"
#include "tiledata.h"
#include "dx11_utils/buffers.h"

class Recon5DGPU : public CReconstruct
{
public:
	Recon5DGPU(const SampleInfo *pSamples, const SampleInfoHeader *pSampleHeader, const float dmin, const float dmax);

	virtual ~Recon5DGPU();

	virtual Image *Reconstruct(float &time);
	virtual Image *ReconstructParallel(float &time);

private:
	//
	// Sample stream header data, shared with GPU side hlsl code
	//
	struct SImageSettings
	{
		UINT Nx;
		UINT Ny;
		UINT Tx;
		UINT Ty;
		UINT spp;
		FLOAT DepthMin;
		FLOAT DepthMax;
		FLOAT Camera_C0;
		FLOAT Camera_C1;
		FLOAT MaxBlur;
		FLOAT sigma_x;
		FLOAT sigma_a;
		FLOAT sigma_t;
		FLOAT pad[3]; // 64 Bytes
	};

	//
	// Filter info for each layer and tile, stored in a temporary buffer and shared between the two algorithm passes
	//
	struct SLayerInfo
	{
		int nSamples;
		float slopet;
		float sigmax;
		float sigmay;
		float sigmat;
		float sigmau;
		float sigmav;
		float shearxu;
		float shearyv;
		float angle;
	};

	ID3D11Device *m_pDevice;
	ID3D11DeviceContext *m_pContext;

	// Compute shader programs
	ID3D11ComputeShader *m_pLayerStatsCS;
	ID3D11ComputeShader *m_pReconstructCS;

	// Sample header information
	CBufferConst<SImageSettings> *m_pImageSettings;

	// Sample input buffers
	CBuffer2DRO<DXGI_FORMAT_R8G8B8A8_UINT> *m_pBufXYUV;    // Sample x,y,u,v	- 8 bit
	CBuffer2DRO<DXGI_FORMAT_R8_UINT> *m_pBufT;             // Sample t			- 8 bit
	CBuffer2DRO<DXGI_FORMAT_R8G8_UINT> *m_pBufVel;         // Sample Vx,Vy		- 8 bit
	CBuffer2DRO<DXGI_FORMAT_R16G16B16A16_FLOAT> *m_pBuffC; // Sample R,G,B		- Float 16
	CBuffer2DRO<DXGI_FORMAT_R32_FLOAT> *m_pBuffZ;          // Sample Z			- Float 32

	// Temporary buffer storing layer statistics - connecting algorithm pass 1 and 2
	CStructuredBuffer1DRW<SLayerInfo> *m_pBufLayers;

	// Output buffer (reconstructed colors per pixel)
	CBuffer2DRW<DXGI_FORMAT_R16G16B16A16_FLOAT> *m_pBufComposite; // Pixel R,G,B,A	- Float 16

	// Helper functions
	void CreateComputeShader(LPCTSTR pFile, ID3D11ComputeShader **shader);
	void RunComputeShader(ID3D11ComputeShader *pComputeShader, UINT nNumViews,
	                      ID3D11ShaderResourceView **pShaderResourceViews, UINT nNumUAVs,
	                      ID3D11UnorderedAccessView **pUnorderedAccessView, UINT X, UINT Y);
	void CopyOutput();
	void CreateComputeDevice();
};
