#pragma once

/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include "config.h"

template <typename T> class CFilter
{
public:
	CFilter(int BaseSizeXY)
	{
		m_Size = BaseSizeXY;
		int TotalSize = BaseSizeXY * BaseSizeXY;
		m_Data = new T[TotalSize];
		memset(m_Data, 0, TotalSize * sizeof(T));
	};

	void Reset()
	{
		int TotalSize = m_Size * m_Size;
		memset(m_Data, 0, TotalSize * sizeof(T));
	};

	void ResetToValue(const T &value)
	{
		int TotalSize = m_Size * m_Size;
		for (int i = 0; i < TotalSize; ++i)
		{
			m_Data[i] = value;
		}
	};

	~CFilter() { delete[] m_Data; };

	inline T &FetchData(int x, int y)
	{
		int index = y * m_Size + x;
		return m_Data[index];
	};

private:
	T *m_Data;
	int m_Size;
};
