/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include "simpleBox.h"
#include <iostream>

Image *SimpleBox::Reconstruct(float &time)
{
	unsigned long t_start = timeGetTime();
	int Nx = m_pSampleHeader->xResolution;
	int Ny = m_pSampleHeader->yResolution;
	int spp = m_pSampleHeader->spp;
	float4 *CBuffer = m_Image.GetBuffer();

	const float wu = -0.5f / (m_sigma_a * m_sigma_a);
	const float wt = -0.5f / (m_sigma_t * m_sigma_t);

	for (int i = 0; i < Ny; i++)
	{
		for (int j = 0; j < Nx; j++)
		{
			float rgba[4] = {0, 0, 0, 1};
			float norm = 0;

			for (int k = 0; k < spp; k++)
			{
				int idx = spp * (j + Nx * i) + k;

				const SampleInfo &Si = m_pSamples[idx];
				float x0 = j + 0.5f;
				float y0 = i + 0.5f;
				float dx = Si.imageX - x0;
				float dy = Si.imageY - y0;
				float rl = Si.lensU * Si.lensU + Si.lensV * Si.lensV;
				float t2 = Si.time * Si.time;
				float w = exp(t2 * wt + rl * wu);

				norm += w;
				rgba[0] += w * Si.rgb[0];
				rgba[1] += w * Si.rgb[1];
				rgba[2] += w * Si.rgb[2];
			}

			CBuffer[(i * Nx + j)] = float4(rgba[0] / norm, rgba[1] / norm, rgba[2] / norm, 1.0f);
		}
	}
	unsigned long t_end = timeGetTime();
	time = (float)(t_end - t_start);
	return &m_Image;
}

Image *SimpleBox::ReconstructParallel(float &time) { return Reconstruct(time); }