/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include "tiledata.h"

TileData::TileData(int baseSize, const std::vector<float2> &depthRanges)
{
	filter = new CFilter<PixelInfo>(baseSize);
	filter1 = new CFilter<PixelInfo>(baseSize);
	filterVis = new CFilter<float>(baseSize);

	filterVis->ResetToValue(1.0f);

	// Create partitions - these will be compacted later
	auto lit = partList.before_begin();
	for (auto &depth : depthRanges)
	{
		lit = partList.emplace_after(lit, depth.x, depth.y); // Create new partition based on depth ranges
	}
}

TileData::~TileData()
{
	delete filter;
	delete filter1;
	delete filterVis;
	for (auto it : partList)
	{
		it.Clear();
	}
	partList.clear();
}

void TileData::Clear()
{
	for (auto it : partList)
	{
		it.Clear();
	}
	filterVis->ResetToValue(1.0f);
}