#pragma once

/* 
* Copyright (c) 2013-2014, Intel Corporation
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
* - Redistributions of source code must retain the above copyright notice,
*   this list of conditions and the following disclaimer.
* - Redistributions in binary form must reproduce the above copyright notice,
*   this list of conditions and the following disclaimer in the documentation
*   and/or other materials provided with the distribution.
* - Neither the name of Intel Corporation nor the names of its contributors
*   may be used to endorse or promote products derived from this software
*   without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
* LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

#include "common.h"
#include "rfilter.h"
#include "partition.h"

#include <forward_list>
#include <vector>

struct PixelInfo
{
	PixelInfo() : col(0), w(0) {}

	inline PixelInfo operator+(const PixelInfo &p1) const 
	{
		PixelInfo p2;
		p2.col = col + p1.col;
		p2.w = w + p1.w;
		return p2;
	};

	inline PixelInfo operator*(const float &f) const 
	{
		PixelInfo p2;
		p2.col = col * f;
		p2.w = w * f;
		return p2;
	};

	inline PixelInfo operator/(const float &f) const 
	{
		PixelInfo p2;
		p2.col = col / f;
		p2.w = w / f;
		return p2;
	};

	inline PixelInfo &operator+=(const PixelInfo &rhs)
	{
		col += rhs.col;
		w   += rhs.w;
		return *this;
	}

	float4 col;
	float w;
};

struct TileData
{
	CFilter<PixelInfo> *filter;
	CFilter<PixelInfo> *filter1;
	CFilter<float> *filterVis;

	std::forward_list<Partition> partList;

	TileData(int baseSize, const std::vector<float2> &depthRanges);
	~TileData();
	void Clear();
};